package dev.shadowsoffire.apotheosis.adventure.affix;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import dev.shadowsoffire.apotheosis.Apotheosis;
import dev.shadowsoffire.apotheosis.adventure.affix.Affix;
import dev.shadowsoffire.apotheosis.adventure.affix.AffixInstance;
import dev.shadowsoffire.apotheosis.adventure.affix.AffixRegistry;
import dev.shadowsoffire.apotheosis.adventure.affix.AffixType;
import dev.shadowsoffire.apotheosis.adventure.loot.LootCategory;
import dev.shadowsoffire.apotheosis.adventure.loot.LootRarity;
import dev.shadowsoffire.apotheosis.adventure.loot.RarityRegistry;
import dev.shadowsoffire.placebo.codec.CodecProvider;
import dev.shadowsoffire.placebo.reload.DynamicHolder;
import dev.shadowsoffire.placebo.util.CachedObject;
import dev.shadowsoffire.placebo.util.StepFunction;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.stream.Stream;
import javax.annotation.Nullable;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.nbt.ListTag;
import net.minecraft.nbt.StringTag;
import net.minecraft.nbt.Tag;
import net.minecraft.network.chat.Component;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.entity.projectile.AbstractArrow;
import net.minecraft.world.item.ItemStack;

public class AffixHelper {
  public static final ResourceLocation AFFIX_CACHED_OBJECT = Apotheosis.loc("affixes");
  
  public static final String DISPLAY = "display";
  
  public static final String LORE = "Lore";
  
  public static final String AFFIX_DATA = "affix_data";
  
  public static final String AFFIXES = "affixes";
  
  public static final String RARITY = "rarity";
  
  public static final String NAME = "name";
  
  public static final String CATEGORY = "category";
  
  public static void applyAffix(ItemStack stack, AffixInstance affix) {
    HashMap<DynamicHolder<? extends Affix>, AffixInstance> affixes = new HashMap<>(getAffixes(stack));
    affixes.put(affix.affix(), affix);
    setAffixes(stack, affixes);
  }
  
  public static void setAffixes(ItemStack stack, Map<DynamicHolder<? extends Affix>, AffixInstance> affixes) {
    CompoundTag afxData = stack.m_41698_("affix_data");
    CompoundTag affixesTag = new CompoundTag();
    for (AffixInstance inst : affixes.values())
      affixesTag.m_128350_(inst.affix().getId().toString(), inst.level()); 
    afxData.m_128365_("affixes", (Tag)affixesTag);
  }
  
  public static void setName(ItemStack stack, Component name) {
    CompoundTag afxData = stack.m_41698_("affix_data");
    afxData.m_128359_("name", Component.Serializer.m_130703_(name));
  }
  
  @Nullable
  public static Component getName(ItemStack stack) {
    if (!stack.m_41782_())
      return null; 
    CompoundTag afxData = stack.m_41737_("affix_data");
    if (afxData == null)
      return null; 
    return (Component)Component.Serializer.m_130701_(afxData.m_128461_("name"));
  }
  
  public static Map<DynamicHolder<? extends Affix>, AffixInstance> getAffixes(ItemStack stack) {
    if (AffixRegistry.INSTANCE.getValues().isEmpty())
      return Collections.emptyMap(); 
    return (Map<DynamicHolder<? extends Affix>, AffixInstance>)CachedObject.CachedObjectSource.getOrCreate(stack, AFFIX_CACHED_OBJECT, dev.shadowsoffire.apotheosis.adventure.affix.AffixHelper::getAffixesImpl, CachedObject.hashSubkey("affix_data"));
  }
  
  public static Map<DynamicHolder<? extends Affix>, AffixInstance> getAffixesImpl(ItemStack stack) {
    if (stack.m_41619_())
      return Collections.emptyMap(); 
    Map<DynamicHolder<? extends Affix>, AffixInstance> map = new HashMap<>();
    CompoundTag afxData = stack.m_41737_("affix_data");
    if (afxData != null && afxData.m_128441_("affixes")) {
      CompoundTag affixes = afxData.m_128469_("affixes");
      DynamicHolder<LootRarity> rarity = getRarity(afxData);
      if (!rarity.isBound())
        rarity = RarityRegistry.getMinRarity(); 
      LootCategory cat = LootCategory.forItem(stack);
      for (String key : affixes.m_128431_()) {
        DynamicHolder<Affix> affix = AffixRegistry.INSTANCE.holder(new ResourceLocation(key));
        if (!affix.isBound() || !((Affix)affix.get()).canApplyTo(stack, cat, (LootRarity)rarity.get()))
          continue; 
        float lvl = affixes.m_128457_(key);
        map.put(affix, new AffixInstance(affix, stack, rarity, lvl));
      } 
    } 
    return Collections.unmodifiableMap(map);
  }
  
  public static Stream<AffixInstance> streamAffixes(ItemStack stack) {
    return getAffixes(stack).values().stream().filter(AffixInstance::isValid);
  }
  
  public static boolean hasAffixes(ItemStack stack) {
    return (stack.m_41782_() && !stack.m_41783_().m_128469_("affix_data").m_128469_("affixes").m_128456_());
  }
  
  public static void setRarity(ItemStack stack, LootRarity rarity) {
    CompoundTag afxData = stack.m_41698_("affix_data");
    afxData.m_128359_("rarity", RarityRegistry.INSTANCE.getKey((CodecProvider)rarity).toString());
  }
  
  public static void copyFrom(ItemStack stack, Entity entity) {
    if (stack.m_41782_() && stack.m_41737_("affix_data") != null) {
      CompoundTag afxData = stack.m_41737_("affix_data").m_6426_();
      afxData.m_128359_("category", LootCategory.forItem(stack).getName());
      entity.getPersistentData().m_128365_("affix_data", (Tag)afxData);
    } 
  }
  
  @Nullable
  public static LootCategory getShooterCategory(Entity entity) {
    CompoundTag afxData = entity.getPersistentData().m_128469_("affix_data");
    if (afxData != null && afxData.m_128441_("category"))
      return LootCategory.byId(afxData.m_128461_("category")); 
    return null;
  }
  
  public static Map<DynamicHolder<? extends Affix>, AffixInstance> getAffixes(AbstractArrow arrow) {
    Map<DynamicHolder<? extends Affix>, AffixInstance> map = new HashMap<>();
    CompoundTag afxData = arrow.getPersistentData().m_128469_("affix_data");
    if (afxData != null && afxData.m_128441_("affixes")) {
      CompoundTag affixes = afxData.m_128469_("affixes");
      DynamicHolder<LootRarity> rarity = getRarity(afxData);
      if (!rarity.isBound())
        rarity = RarityRegistry.getMinRarity(); 
      for (String key : affixes.m_128431_()) {
        DynamicHolder<Affix> affix = AffixRegistry.INSTANCE.holder(new ResourceLocation(key));
        if (!affix.isBound())
          continue; 
        float lvl = affixes.m_128457_(key);
        map.put(affix, new AffixInstance(affix, ItemStack.f_41583_, rarity, lvl));
      } 
    } 
    return map;
  }
  
  public static Stream<AffixInstance> streamAffixes(AbstractArrow arrow) {
    return getAffixes(arrow).values().stream();
  }
  
  public static DynamicHolder<LootRarity> getRarity(ItemStack stack) {
    if (!stack.m_41782_())
      return RarityRegistry.INSTANCE.emptyHolder(); 
    CompoundTag afxData = stack.m_41737_("affix_data");
    return getRarity(afxData);
  }
  
  public static DynamicHolder<LootRarity> getRarity(@Nullable CompoundTag afxData) {
    if (afxData != null)
      try {
        return RarityRegistry.byLegacyId(afxData.m_128461_("rarity"));
      } catch (IllegalArgumentException e) {
        afxData.m_128473_("rarity");
        return RarityRegistry.byLegacyId("empty");
      }  
    return RarityRegistry.INSTANCE.emptyHolder();
  }
  
  public static Collection<DynamicHolder<Affix>> byType(AffixType type) {
    return AffixRegistry.INSTANCE.getTypeMap().get(type);
  }
  
  public static StepFunction step(float min, int steps, float step) {
    return new StepFunction(min, steps, step);
  }
  
  public static void addLore(ItemStack stack, Component lore) {
    CompoundTag display = stack.m_41698_("display");
    ListTag tag = display.m_128437_("Lore", 8);
    tag.add(StringTag.m_129297_(Component.Serializer.m_130703_(lore)));
    display.m_128365_("Lore", (Tag)tag);
  }
}
